/**
 * @file main.c Main file
 *
 * Copyright (C) 2004-2005 Christian Hammond.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA  02111-1307  USA
 */
#include <glib.h>
#include <libgalago/galago.h>
#include <libedata-book/Evolution-DataServer-Addressbook.h>

#include <libebook/e-book.h>
#include <libedata-book/e-data-book-factory.h>
#include <bonobo/bonobo-main.h>

#include <string.h>

#define E_DATA_BOOK_FACTORY_OAF_ID \
    "OAFIID:GNOME_Evolution_DataServer_BookFactory"

static EBookView *book_view = NULL;
static EBook *book = NULL;
static gulong book_view_tag = 0;
static GalagoService *aim, *groupwise, *jabber, *yahoo, *msn, *icq;

static void
add_accounts(GalagoPerson *person, EContact *contact,
			 GalagoService *service, EContactField field)
{
	GalagoAccount *account;
	GList *im_attr_list, *l;

	im_attr_list = e_contact_get_attributes(contact, field);

	for (l = im_attr_list; l != NULL; l = l->next)
	{
		EVCardAttribute *attr = (EVCardAttribute *)l->data;
		char *username;

		username = e_vcard_attribute_get_value(attr);

		galago_log(GALAGO_LOG_INFO, "Adding account %s on %s\n",
				   username, galago_service_get_id(service));

		account = galago_account_new(service, person, username);
	}
}

static void
add_remove_accounts(GalagoPerson *person, EContact *contact,
					GalagoService *service, EContactField field)
{
	GalagoAccount *account;
	const GalagoList *gl, *gl_next;
	GList *im_attr_list, *l;

	im_attr_list = e_contact_get_attributes(contact, field);

	for (l = im_attr_list; l != NULL; l = l->next)
	{
		EVCardAttribute *attr = (EVCardAttribute *)l->data;
		char *username;

		username = e_vcard_attribute_get_value(attr);

		if (galago_person_get_account(person, service, username, FALSE))
		{
			galago_log(GALAGO_LOG_INFO, "Adding account %s on %s\n",
					   username, galago_service_get_id(service));

			account = galago_account_new(service, person, username);
		}
	}

	for (gl = galago_person_get_accounts(person, FALSE);
		 gl != NULL;
		 gl = gl_next)
	{
		gboolean found = FALSE;
		const char *username1, *username2;

		gl_next = gl->next;

		account = (GalagoAccount *)gl->data;

		if (galago_account_get_service(account) != service)
			continue;

		username1 = galago_account_get_username(account);

		for (l = im_attr_list; l != NULL; l = l->next)
		{
			EVCardAttribute *attr = (EVCardAttribute *)l->data;

			username2 = e_vcard_attribute_get_value(attr);

			if (!strcmp(username1, username2))
			{
				found = TRUE;
				break;
			}
		}

		if (!found)
		{
			galago_log(GALAGO_LOG_INFO, "Removing account %s on %s\n",
					   galago_account_get_username(account),
					   galago_service_get_id(service));
			galago_object_unref(account);
		}
	}
}

static void
set_properties(GalagoPerson *person, EContact *contact)
{
	EContactName *contact_name;
	const EContactPhoto *ephoto;

	contact_name = e_contact_get(contact, E_CONTACT_NAME);

	if (contact_name == NULL)
		return;

	if (contact_name->given != NULL && *contact_name->given != '\0')
	{
		galago_person_set_property_string(person,
										  GALAGO_PERSON_PROP_FIRST_NAME,
										  contact_name->given);
	}

	if (contact_name->family != NULL && *contact_name->family != '\0')
	{
		galago_person_set_property_string(person,
										  GALAGO_PERSON_PROP_LAST_NAME,
										  contact_name->family);
	}

	e_contact_name_free(contact_name);

	ephoto = e_contact_get(contact, E_CONTACT_PHOTO);

	if (ephoto == NULL)
		galago_person_set_photo(person, NULL);
	else
	{
		galago_photo_new(person, (unsigned char *)ephoto->data,
						 ephoto->length);
	}
}

static void
contacts_changed_cb(EBookView *book_view, const GList *contacts)
{
	const GList *l;

	for (l = contacts; l != NULL; l = l->next)
	{
		EContact *contact = (EContact *)l->data;
		GalagoPerson *person;
		const char *uid;

		uid = e_contact_get_const(contact, E_CONTACT_UID);

		person = galago_person_new(uid, TRUE);

		set_properties(person, contact);

		add_remove_accounts(person, contact, aim,       E_CONTACT_IM_AIM);
		add_remove_accounts(person, contact, groupwise, E_CONTACT_IM_GROUPWISE);
		add_remove_accounts(person, contact, jabber,    E_CONTACT_IM_JABBER);
		add_remove_accounts(person, contact, yahoo,     E_CONTACT_IM_YAHOO);
		add_remove_accounts(person, contact, msn,       E_CONTACT_IM_MSN);
		add_remove_accounts(person, contact, icq,       E_CONTACT_IM_ICQ);
	}
}

static void
contacts_added_cb(EBookView *book_view, const GList *contacts)
{
	const GList *l;

	for (l = contacts; l != NULL; l = l->next)
	{
		EContact *contact = (EContact *)l->data;
		GalagoPerson *person;
		const char *uid;

		uid = e_contact_get_const(contact, E_CONTACT_UID);

		person = galago_person_new(uid, TRUE);

		set_properties(person, contact);

		add_accounts(person, contact, aim,       E_CONTACT_IM_AIM);
		add_accounts(person, contact, groupwise, E_CONTACT_IM_GROUPWISE);
		add_accounts(person, contact, jabber,    E_CONTACT_IM_JABBER);
		add_accounts(person, contact, yahoo,     E_CONTACT_IM_YAHOO);
		add_accounts(person, contact, msn,       E_CONTACT_IM_MSN);
		add_accounts(person, contact, icq,       E_CONTACT_IM_ICQ);
	}
}

static void
got_book_view_cb(EBook *book, EBookStatus status, EBookView *view,
				 gpointer user_data)
{
	book_view_tag = 0;

	if (status != E_BOOK_ERROR_OK)
	{
		fprintf(stderr, "Unable to retrieve book view!\n");
		exit(1);
	}

	book_view = view;

	g_object_ref(book_view);

	g_signal_connect(G_OBJECT(book_view), "contacts_changed",
					 G_CALLBACK(contacts_changed_cb), book);
	g_signal_connect(G_OBJECT(book_view), "contacts_added",
					 G_CALLBACK(contacts_added_cb), book);

	e_book_view_start(view);
}

static void
setup_book_view(void)
{
	EBookQuery *query;

	book = e_book_new_system_addressbook(NULL);

	if (book == NULL)
	{
		fprintf(stderr, "Failed to create local addressbook\n");

		exit(1);
	}

	if (!e_book_open(book, FALSE, NULL))
	{
		fprintf(stderr, "Failed to open local addressbook\n");

		g_object_unref(book);

		exit(1);
	}

	query = e_book_query_any_field_contains("");

	book_view_tag = e_book_async_get_book_view(book, query, NULL, -1,
											   got_book_view_cb, NULL);
	e_book_query_unref(query);
}

static void
setup_services(void)
{
	aim       = galago_service_new(GALAGO_SERVICE_ID_AIM,       NULL, TRUE, 0);
	groupwise = galago_service_new(GALAGO_SERVICE_ID_GROUPWISE, NULL, TRUE, 0);
	jabber    = galago_service_new(GALAGO_SERVICE_ID_JABBER,    NULL, TRUE, 0);
	yahoo     = galago_service_new(GALAGO_SERVICE_ID_YAHOO,     NULL, TRUE, 0);
	msn       = galago_service_new(GALAGO_SERVICE_ID_MSN,       NULL, TRUE, 0);
	icq       = galago_service_new(GALAGO_SERVICE_ID_ICQ,       NULL, TRUE, 0);
}

int
main(int argc, char **argv)
{
	GMainLoop *loop;

	loop = g_main_loop_new(NULL, FALSE);

	if (!galago_glib_init("eds-feed", TRUE, NULL))
	{
		fprintf(stderr, "Unable to initialize Galago.\n");
		exit(1);
	}

	galago_core_set_exit_with_daemon(TRUE);

	if (!bonobo_init_full(NULL, NULL, bonobo_activation_orb_get(),
						  CORBA_OBJECT_NIL, CORBA_OBJECT_NIL))
	{
		fprintf(stderr, "Unable to initialize bonobo.\n");
		exit(1);
	}

	bonobo_activate();

	setup_book_view();
	setup_services();

	g_main_loop_run(loop);

	return 0;
}
